@@ -402,4 +402,10 @@ create table jaaql__email_history (
     encrypted_recipients text,
     encrypted_body text,
     encrypted_attachments text
+);
+
+create table jaaql__template_validation_table (
+    template_path varchar(255),
+    validation_table varchar(255),
+    PRIMARY KEY(template_path)
 );
\ No newline at end of file
